@@ -1,9 +1,8 @@
 #!/usr/bin/env python3
-"""
-Example for rewriting response headers by middleware.
-"""
+"""Example for rewriting response headers by middleware."""
 
 from aiohttp import web
+from aiohttp.typedefs import Handler
 
 
 async def handler(request):
@@ -11,19 +10,19 @@ async def handler(request):
 
 
 @web.middleware
-async def middleware(request, handler):
+async def middleware(request: web.Request, handler: Handler) -> web.StreamResponse:
     try:
         response = await handler(request)
     except web.HTTPException as exc:
         raise exc
     if not response.prepared:
-        response.headers['SERVER'] = "Secured Server Software"
+        response.headers["SERVER"] = "Secured Server Software"
     return response
 
 
 def init():
     app = web.Application(middlewares=[middleware])
-    app.router.add_get('/', handler)
+    app.router.add_get("/", handler)
     return app
 
 
