@@ -2,6 +2,7 @@
 
 import io
 import re
+import time
 from hashlib import md5, sha1
 from typing import Generator, Literal, Union
 from unittest import mock
@@ -12,6 +13,7 @@ from yarl import URL
 from aiohttp import ClientSession, hdrs
 from aiohttp.client_exceptions import ClientError
 from aiohttp.client_middleware_digest_auth import (
+    _HEADER_PAIRS_PATTERN,
     DigestAuthChallenge,
     DigestAuthMiddleware,
     DigestFunctions,
@@ -1326,3 +1328,15 @@ async def test_case_sensitive_algorithm_server(
     assert request_count == 2  # Initial 401 + successful retry
     assert len(auth_algorithms) == 1
     assert auth_algorithms[0] == "MD5-sess"  # Not "MD5-SESS"
+
+
+def test_regex_performance() -> None:
+    value = "0" * 54773 + "\\0=a"
+    start = time.perf_counter()
+    matches = _HEADER_PAIRS_PATTERN.findall(value)
+    end = time.perf_counter()
+
+    # If this is taking more than 10ms, there's probably a performance/ReDoS issue.
+    assert (end - start) < 0.01
+    # This example probably shouldn't produce a match either.
+    assert not matches
