@@ -3,7 +3,6 @@ import bz2
 import gzip
 import pathlib
 import socket
-import zlib
 from typing import Any, Iterable, Optional
 from unittest import mock
 
@@ -11,6 +10,8 @@ import pytest
 
 import aiohttp
 from aiohttp import web
+from aiohttp.compression_utils import ZLibBackend
+from aiohttp.pytest_plugin import AiohttpClient
 
 try:
     import brotlicffi as brotli
@@ -300,6 +301,7 @@ async def test_static_file_custom_content_type_compress(
     [("gzip, deflate", "gzip"), ("gzip, deflate, br", "br")],
 )
 @pytest.mark.parametrize("forced_compression", [None, web.ContentCoding.gzip])
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_static_file_with_encoding_and_enable_compression(
     hello_txt: pathlib.Path,
     aiohttp_client: Any,
@@ -612,6 +614,7 @@ async def test_static_file_ssl(
 
     await resp.release()
     await client.close()
+    await conn.close()
 
 
 async def test_static_file_directory_traversal_attack(aiohttp_client) -> None:
@@ -635,13 +638,16 @@ async def test_static_file_directory_traversal_attack(aiohttp_client) -> None:
 
     url_abspath = "/static/" + str(full_path.resolve())
     resp = await client.get(url_abspath)
-    assert 403 == resp.status
+    assert resp.status == 404
     await resp.release()
 
     await client.close()
 
 
-async def test_static_file_huge(aiohttp_client, tmp_path) -> None:
+@pytest.mark.skip_blockbuster
+async def test_static_file_huge(
+    aiohttp_client: AiohttpClient, tmp_path: pathlib.Path
+) -> None:
     file_path = tmp_path / "huge_data.unknown_mime_type"
 
     # fill 20MB file
@@ -1047,6 +1053,7 @@ async def test_static_file_if_range_invalid_date(
     await client.close()
 
 
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_static_file_compression(aiohttp_client, sender) -> None:
     filepath = pathlib.Path(__file__).parent / "data.unknown_mime_type"
 
@@ -1061,7 +1068,7 @@ async def test_static_file_compression(aiohttp_client, sender) -> None:
 
     resp = await client.get("/")
     assert resp.status == 200
-    zcomp = zlib.compressobj(wbits=zlib.MAX_WBITS)
+    zcomp = ZLibBackend.compressobj(wbits=ZLibBackend.MAX_WBITS)
     expected_body = zcomp.compress(b"file content\n") + zcomp.flush()
     assert expected_body == await resp.read()
     assert "application/octet-stream" == resp.headers["Content-Type"]
@@ -1071,7 +1078,10 @@ async def test_static_file_compression(aiohttp_client, sender) -> None:
     await client.close()
 
 
-async def test_static_file_huge_cancel(aiohttp_client, tmp_path) -> None:
+@pytest.mark.skip_blockbuster
+async def test_static_file_huge_cancel(
+    aiohttp_client: AiohttpClient, tmp_path: pathlib.Path
+) -> None:
     file_path = tmp_path / "huge_data.unknown_mime_type"
 
     # fill 100MB file
