@@ -232,13 +232,11 @@ def test_parse_frame_header_control_frame(
         raise out.exception()
 
 
-@pytest.mark.xfail()
-def test_parse_frame_header_new_data_err(
-    out: WebSocketDataQueue, parser: PatchableWebSocketReader
-) -> None:
-    with pytest.raises(WebSocketError):
-        parser.parse_frame(struct.pack("!BB", 0b000000000, 0b00000000))
-        raise out.exception()
+def test_parse_frame_header_new_data_err(parser: PatchableWebSocketReader) -> None:
+    with pytest.raises(WebSocketError) as msg:
+        parser._feed_data(struct.pack("!BB", 0b00000000, 0b00000000))
+    assert msg.value.code == WSCloseCode.PROTOCOL_ERROR
+    assert str(msg.value) == "Continuation frame for non started message"
 
 
 def test_parse_frame_header_payload_size(
